/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * application.cpp - commandline and settings reader and controller in mvc like architecture
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "application.h"
#include "mainwindow.h"

// FIXME: take care of pingtimeout

App::App( os::OptionParser* pcOpts ) : os::Application( "application/x-vnd.kasbah-goIRC" )
{
	m_cNick = os::String();
	m_cServer = os::String();
	m_nPort = 0;
	m_cPassword = os::String();

	{  // read the settings from commandline, or if not given, from settings file
		os::OptionParser::option const* pcOption;

		os::Settings cSettings = os::Settings();
		cSettings.Load();

		// FIXME: in contrast to the documentation the OptionParser does not recognize values for options in the shortform
		// "Both short-options and long-options can have additional arguments. If a short-option take an additional argument
		//  it the next option will be used as the argument value."
		// e.g. -n kasbah does NOT work while --nick=kasbah does work
		if( ( pcOption = pcOpts->FindOption( 'n' ) ) != 0 && pcOption->has_value )
			m_cNick = pcOption->value.const_str();
		else
			m_cNick = cSettings.GetString( "nick" );

		if( ( pcOption = pcOpts->FindOption( 's' ) ) != 0 && pcOption->has_value )
			m_cServer = pcOption->value.const_str();
		else
			m_cServer = cSettings.GetString( "server" );

		if( ( pcOption = pcOpts->FindOption( 'p' ) ) != 0 && pcOption->has_value )
			m_nPort = atoi( pcOption->value.c_str() );
		else
			m_nPort = cSettings.GetInt16( "port", 0 );

		if( ( pcOption = pcOpts->FindOption( 'P' ) ) != 0 && pcOption->has_value )
			m_cPassword = pcOption->value.const_str();
		else
			m_cPassword = cSettings.GetString( "password" );

		delete( pcOption );

		// get hardwired default values, if nothing was set
		// if no nick was given, use the login name from the os environment
		if( m_cNick.empty() )
		{
			m_cNick = getenv( "USER" );
			// only morons are using an irc client with a root account :P
			// i do it as of writing this too, so don't take it serious, it's a joke! ;)
			if( m_cNick == "root" ) m_cNick = "moron";
		}

		// if no server was given, connect to freenode (#syllable waits for YOU to join ;) )
		if( m_cServer.empty() )
			m_cServer = "irc.freenode.net";

		// if no port was given, use default irc port
		if( m_nPort == 0 )
			m_nPort = 6667;
	}

	// create the window
	m_pcMainWindow = new MainWindow();
	m_pcMainWindow->CenterInScreen();
	m_pcMainWindow->Show();
	m_pcMainWindow->MakeFocus();

	// initialize the Main Window Messenger
	m_cMessengerWindowMain = os::Messenger( m_pcMainWindow );

	// start the socket Thread
	m_pcSocketThread = new SocketThread();
	m_pcSocketThread->Run();
	
	// initialize the Socket Messenger
	m_cMessengerSocket = os::Messenger( m_pcSocketThread );

	// connect the socket to a server
	os::Message cMessage( M_SOCKET_CONNECT );
	cMessage.AddString( "server", m_cServer );
	cMessage.AddInt16( "port", m_nPort );
	m_cMessengerSocket.SendMessage( &cMessage );
}

void App::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_APP_SEND:
		{
			os::String cString = os::String();
			pcMessage->FindString( "send", &cString );

			os::Message cMessageSend = os::Message( M_SOCKET_SEND );
			cMessageSend.AddString( "send", cString );
			m_cMessengerSocket.SendMessage( &cMessageSend );
		}
		break;

		case M_SOCKET_CONNECTED:
		{
			os::Message cMessageSocket( M_SOCKET_SEND );

			if( !m_cPassword.empty() )
				cMessageSocket.AddString( "send", os::String().Format( "PASS %s", m_cPassword.c_str() ) );

			cMessageSocket.AddString( "send", os::String().Format( "NICK %s", m_cNick.c_str() ) );
			cMessageSocket.AddString(
				"send", os::String().Format( "USER %s localhost %s :%s", m_cNick.c_str(), m_cServer.c_str(), m_cNick.c_str() )
			);
			m_cMessengerSocket.SendMessage( &cMessageSocket );
		}
		break;

		case M_SOCKET_ERROR:
		case M_SOCKET_SENT:
		case M_SOCKET_RECEIVED:
		case M_SOCKET_INFO:
		{
			os::String cString = os::String();
			os::String cOutput = os::String();

			switch( pcMessage->GetCode() )
			{				
				case M_SOCKET_ERROR:
				{
					int8 nSocketError;
					pcMessage->FindInt8( "errornumber", &nSocketError );
					cOutput = os::String().Format( "!! Socket Error #%d\n", nSocketError ).c_str();
				}
				break;

				case M_SOCKET_INFO:
				{
					pcMessage->FindString( "info", &cString );
					cOutput = os::String().Format( "** %s\n", cString.c_str() ).c_str();
				}
				break;

				case M_SOCKET_SENT:
				{
					pcMessage->FindString( "sent", &cString );
					cOutput = os::String().Format( "<< %s\n", cString.c_str() ).c_str();
				}
				break;

				case M_SOCKET_RECEIVED:
				{
					pcMessage->FindString( "received", &cString);
					cOutput = os::String().Format( ">> %s\n", cString.c_str() ).c_str();

					// reply to ping with pong
					if( 0 == cString.substr( 0, 4 ).Compare( "PING" ) )
					{
						os::Message cMessageSocket( M_SOCKET_SEND );
						cMessageSocket.AddString( "send", os::String( "PONG" ) );
						m_cMessengerSocket.SendMessage( &cMessageSocket );
					}
				}
				break;
			}

			os::Message cMessageWindow( M_APP_DISPLAY );
			cMessageWindow.AddString( "display", cOutput.const_str() );
			m_cMessengerWindowMain.SendMessage( &cMessageWindow );
		}
		break;
		
		case M_APP_QUIT:
		{
			m_cMessengerSocket.SendMessage( os::M_QUIT );
			PostMessage( os::M_QUIT );
		}
		break;
		
		default:
		{
			std::cout << "not handled Message in Application: " << pcMessage->GetCode() << std::endl;
			Handler::HandleMessage( pcMessage );
		}
		break;
	}
}

