/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * mainwindow.cpp - view component in the mvc like architecture
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "application.h"
#include "mainwindow.h"
#include "messages.h"
#include "settingsdialog.h"

MainWindow::MainWindow() : os::Window( os::Rect( 0, 0, 600, 400 ), "main_wnd", "goIRC - Good Old Internet Relay Chat" )
{
	// fill the mainwindow with the widgets
	os::LayoutView* pcView = new os::LayoutView( GetBounds(), "layout_view" );
	#include "mainwindowLayout.cpp"
	pcView->SetRoot( m_pcRoot );
	AddChild( pcView );

	// set the icon
	os::Resources cCol( get_image_id() );
	os::ResStream *pcStream = cCol.GetResourceStream( "icon48x48.png" );
	os::BitmapImage *pcIcon = new os::BitmapImage( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );
	
	// initialize the Application Messenger
	m_cMessengerApplication = os::Messenger( dynamic_cast<os::Handler*>( os::Application::GetInstance() ) );
}

void MainWindow::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_APP_DISPLAY:
		{
			os::String cString;
			pcMessage->FindString( "display", &cString );

			// add new text at the end of the TextView buffer
			os::TextView::buffer_type cBuffer = m_pcTextView->GetBuffer();
			m_pcTextView->Insert( os::IPoint( cBuffer[cBuffer.size()-1].size(), cBuffer.size()-1 ), cString.c_str() );
		}
		break;

		case M_APP_SEND:
			// don't do anything, just give this message to the controller
			m_cMessengerApplication.SendMessage( pcMessage );
			break;
		
		case M_APP_ABOUT:
		{
			os::Alert* pcAbout = new os::Alert(
				"About goIRC",
				os::String( "goIRC " ) + GOIRC_VERSION + "\nGood Old Internet Relay Chat\n\nA simple IRC Client for Syllable\n\nWritten by Sebastian \"kasbah\" Kasparek, 2007\n\ngoIRC is released under the Gnu Public Licencse (GPL)\nPlease see the file COPYING, distributed with goIRC,\nfor more information\n",
				os::Alert::ALERT_INFO, 0x00, "Ok", NULL );
			pcAbout->Go( NULL );
		}
		break;
		
		case M_APP_SETTINGS:
		{
			// create the settings dialog
			SettingsDialog* pcSettingsDialog = new SettingsDialog();
			pcSettingsDialog->CenterInScreen();
			pcSettingsDialog->Show();
			pcSettingsDialog->MakeFocus();
		}
		break;

		case M_APP_QUIT:
			PostMessage( os::M_QUIT );
			break;

		default:
			std::cout << "not handled Message in MainWindow: " << pcMessage->GetCode() << std::endl;
			Handler::HandleMessage( pcMessage );
			break;
	}
}

bool MainWindow::OkToQuit()
{
	// just quit, the controller will take care of closing connections
	m_cMessengerApplication.SendMessage( M_APP_QUIT );
	return( true );
}

