/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * settingsdialog.cpp - settings dialog combining mvc in one class
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "settingsdialog.h"

SettingsDialog::SettingsDialog() : os::Window( os::Rect( 0, 0, 320, 200 ), "settings_wnd", "goIRC - Settings" )
{
	os::Settings cSettings = os::Settings();
	cSettings.Load();

	os::LayoutView* pcView = new os::LayoutView( GetBounds(), "layout_view" );
	
	m_pcRoot = new os::VLayoutNode( "Root" );
	
	m_pcHLayoutServer = new os::HLayoutNode( "HLayoutServer", 0.1 );
	m_pcHLayoutServer->SetBorders( os::Rect( 10.0, 10.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutServer );
	
	m_pcStringViewServer = new os::StringView( os::Rect(), "StringViewServer", "Server" );
	m_pcHLayoutServer->AddChild( m_pcStringViewServer, 0.1 );
	
	m_pcHLayoutSpacerServer = new os::HLayoutSpacer( "HLayoutSpacerServer", 5.0, 5.0, NULL, 0.1 );
	m_pcHLayoutServer->AddChild( m_pcHLayoutSpacerServer );
	
	m_pcTextViewServer = new os::TextView( os::Rect(), "TextViewServer", cSettings.GetString( "server" ).c_str() );
	m_pcTextViewServer->SetTabOrder( 0 );
	m_pcHLayoutServer->AddChild( m_pcTextViewServer, 1.0 );
	
	m_pcHLayoutNodePort = new os::HLayoutNode( "HLayoutNodePort", 0.1 );
	m_pcHLayoutNodePort->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutNodePort );
	
	m_pcStringViewPort = new os::StringView( os::Rect(), "StringViewPort", "Port" );
	m_pcHLayoutNodePort->AddChild( m_pcStringViewPort, 0.1 );
	
	m_pcHLayoutSpacerPort = new os::HLayoutSpacer( "HLayoutSpacerPort", 5.0, 5.0, NULL, 0.1 );
	m_pcHLayoutNodePort->AddChild( m_pcHLayoutSpacerPort );
	
	m_pcSpinnerPort = new os::Spinner( os::Rect(), "SpinnerPort", cSettings.GetInt16( "port", 0 ), NULL );
	m_pcSpinnerPort->SetMinMax( 0.0, 65535.0 );
	m_pcSpinnerPort->SetStep( 1.0 );
	m_pcSpinnerPort->SetFormat( "%5.f" );
	m_pcSpinnerPort->SetTabOrder( 1 );
	m_pcHLayoutNodePort->AddChild( m_pcSpinnerPort, 1.0 );
	
	m_pcHLayoutNodeNick = new os::HLayoutNode( "HLayoutNodeNick", 0.1 );
	m_pcHLayoutNodeNick->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutNodeNick );
	
	m_pcStringViewNick = new os::StringView( os::Rect(), "StringViewNick", "Nick" );
	m_pcHLayoutNodeNick->AddChild( m_pcStringViewNick, 0.1 );
	
	m_pcHLayoutSpacerNick = new os::HLayoutSpacer( "HLayoutSpacerNick", 5.0, 5.0, NULL, 0.1 );
	m_pcHLayoutNodeNick->AddChild( m_pcHLayoutSpacerNick );
	
	m_pcTextViewNick = new os::TextView( os::Rect(), "TextViewNick", cSettings.GetString( "nick" ).c_str() );
	m_pcTextViewNick->SetTabOrder( 2 );
	m_pcHLayoutNodeNick->AddChild( m_pcTextViewNick, 1.0 );

	m_pcHLayoutNodePassword = new os::HLayoutNode( "HLayoutNodePassword", 0.1 );
	m_pcHLayoutNodePassword->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutNodePassword );
	
	m_pcStringViewPassword = new os::StringView( os::Rect(), "StringViewPassword", "Server Password" );
	m_pcHLayoutNodePassword->AddChild( m_pcStringViewPassword, 0.1 );
	
	m_pcHLayoutSpacerPassword = new os::HLayoutSpacer( "HLayoutSpacerPassword", 5.0, 5.0, NULL, 1.0 );
	m_pcHLayoutNodePassword->AddChild( m_pcHLayoutSpacerPassword );
	
	m_pcTextViewPassword = new os::TextView( os::Rect(), "TextViewPassword", cSettings.GetString( "password" ).c_str() );
	m_pcTextViewPassword->SetTabOrder( 3 );
	m_pcHLayoutNodePassword->AddChild( m_pcTextViewPassword, 1.0 );

	m_pcVLayoutSpacerButtons = new os::VLayoutSpacer( "VLayoutSpacerButtons", 5.0, 5000.0, NULL, 1.0 );
	m_pcRoot->AddChild( m_pcVLayoutSpacerButtons );

	m_pcHLayoutNodeSeparator = new os::HLayoutNode( "HLayoutNodeSeparator", 0.1 );
	m_pcHLayoutNodeSeparator->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutNodeSeparator );

	m_pcSeparator = new os::Separator( os::Rect(), "Separator", os::HORIZONTAL );
	m_pcHLayoutNodeSeparator->AddChild( m_pcSeparator, 1.0 );

	m_pcHLayoutNodeButtons = new os::HLayoutNode( "HLayoutNodeButtons", 0.1 );
	m_pcHLayoutNodeButtons->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	m_pcRoot->AddChild( m_pcHLayoutNodeButtons );

	m_pcHLayoutSpacerPreButtons = new os::HLayoutSpacer( "HLayoutSpacerPreButtons", 0.0, 5000.0, NULL, 1.0 );
	m_pcHLayoutNodeButtons->AddChild( m_pcHLayoutSpacerPreButtons );
	
	m_pcButtonOk = new os::Button( os::Rect(), "ButtonOk", "Ok", new os::Message( M_SETTINGS_SAVE ) );
	m_pcButtonOk->SetTabOrder( 4 );
	m_pcHLayoutNodeButtons->AddChild( m_pcButtonOk, 0.0 );

	m_pcHLayoutSpacerButtons = new os::HLayoutSpacer( "HLayoutSpacerButtons", 5.0, 5.0, NULL, 0.1 );
	m_pcHLayoutNodeButtons->AddChild( m_pcHLayoutSpacerButtons );

	m_pcButtonCancel = new os::Button( os::Rect(), "ButtonCancel", "Cancel", new os::Message( os::M_QUIT ) );
	m_pcButtonCancel->SetTabOrder( 5 );
	m_pcHLayoutNodeButtons->AddChild( m_pcButtonCancel, 0.0 );

// FIXME: get this doing what i want
//	m_pcRoot->SameWidth( "TextViewServer", "SpinnerPort", "TextViewNick", "TextViewPassword", NULL );
	
	pcView->SetRoot( m_pcRoot );
	AddChild( pcView );
	
	// set the icon
	os::Resources cCol( get_image_id() );
	os::ResStream *pcStream = cCol.GetResourceStream( "icon48x48.png" );
	os::BitmapImage *pcIcon = new os::BitmapImage( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );
}

void SettingsDialog::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_SETTINGS_SAVE:
		{
			os::Settings cSettings = os::Settings();
			cSettings.SetString( "server", m_pcTextViewServer->GetValue().AsString() );
			cSettings.SetInt16( "port", m_pcSpinnerPort->GetValue().AsInt16() );
			cSettings.SetString( "nick", m_pcTextViewNick->GetValue().AsString() );
			cSettings.SetString( "password", m_pcTextViewPassword->GetValue().AsString() );
			cSettings.Save();
			PostMessage( os::M_QUIT );
		}
		break;

		default:
			std::cout << "not handled Message in SettingsDialog: " << pcMessage->GetCode() << std::endl;
			Handler::HandleMessage( pcMessage );
			break;
	}
}

bool SettingsDialog::OkToQuit()
{
	// FIXME: add warning when there are unsaved changes
	return( true );
}

