/**
 * SocketThread (C)opyright 2007 Sebastian Kasparek
 *
 * more information is in the doc/SocketReadMe.txt file
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details. 
 *
 * You should have received a copy of the GNU Library General Public 
 * License along with this library; if not, write to the Free 
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#ifndef SOCKET_H
#define SOCKET_H

#include <iostream>

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>

#include <util/application.h>
#include <util/looper.h>
#include <util/message.h>
#include <util/messenger.h>
#include <util/string.h>

// messages
enum
{
	M_SOCKET_CONNECT   = 100,
	M_SOCKET_CONNECTED = 101,
	M_SOCKET_SEND      = 102,
	M_SOCKET_SENT      = 103,
	M_SOCKET_RECEIVED  = 104,
	M_SOCKET_ERROR     = 105,
	M_SOCKET_INFO      = 106,
	M_READSOCKET_ERROR = 107
};

// error numbers
enum
{
	M_SOCKETERROR_DEAD,
	M_SOCKETERROR_COULDNOTOPEN,
	M_SOCKETERROR_SERVERNOTFOUND,
	M_SOCKETERROR_COULDNOTCONNECT,
	M_SOCKETERROR_COULDNOTWRITE,
	M_SOCKETERROR_NOTALLWRITTEN,
	M_SOCKETERROR_SELECTFAILED
};

// start an extra thread for reading from the socket
class SocketReadThread : public os::Looper
{
	public:
		SocketReadThread( const os::Messenger& cSocketMessenger, const int hSocket );
		virtual bool Idle();
		virtual void HandleMessage( os::Message* pcMessage );
	private:
		void PostError( const int8 nError );
		os::Messenger m_cMessengerSocket;
		int m_hSocket;
		fd_set m_sFDSet;
		struct timeval m_sTimeval;
		char m_pzBuffer[4096];
};

class SocketThread : public os::Looper
{
	public:
		SocketThread();
		bool OpenSocket( const os::String* pcServer, const int nPort );
		void CloseSocket();
		virtual void HandleMessage( os::Message* pcMessage );
		virtual bool OkToQuit();
	private:
		void PostErrorCode( const int8 nError );
		void PostStringMessage( const int nMessageCode, const char* pzName, const os::String* );
		int m_hSocket;
		os::String m_cServer;
		int m_nPort;
		SocketReadThread* m_pcSocketReadThread;
		os::Messenger m_cMessenger;
		os::Messenger m_cMessengerSocketRead;
};

#endif

