/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * textinputview.cpp - TextView extended to be usable as input line
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "textinputview.h"

TextInputView::TextInputView( const os::Rect& cLayoutBounds, char* pzTitle, char* pzBuffer, os::Messenger cMessenger )
	:TextView ( cLayoutBounds, pzTitle, pzBuffer )
{
	m_cMessenger = cMessenger;
}

// filter keys and do some action when shortcut was pressed
void TextInputView::KeyDown( const char* pzString, const char* pzRawString, const uint32 nQualifiers )
{
	os::String* pcString = new os::String( pzRawString );
	
	// enter pressed means send the value of the input field
	if( pcString->Compare( "\n" ) == 0 )
	{
		os::Message cMessage( M_APP_SEND );
		cMessage.AddString( "send", this->GetValue().AsString() );
		m_cMessenger.SendMessage( &cMessage );
		this->Clear();
		return;
	}

	// no modifier key or only shift pressed, give it to the textview
	if( !nQualifiers || ( ( nQualifiers & os::QUAL_SHIFT ) && !( nQualifiers & ( os::QUAL_CTRL | os::QUAL_ALT ) ) ) )
	{
		TextView::KeyDown( pzString, pzRawString, nQualifiers );
		return;
	}
	// ctrl pressed, send the Shortcut Messages, if one is associated
	// XXX: create one general shortcut structure?
	if( ( nQualifiers & os::QUAL_CTRL ) && !( nQualifiers & ( os::QUAL_SHIFT | os::QUAL_ALT ) ) )
	{
		if ( pcString->Compare( "q" ) == 0 || pcString->Compare( "Q" ) == 0 )
			m_cMessenger.SendMessage( M_APP_QUIT );
		// send ctrl+c, ctrl+v, ctrl+x to the TextView to let it do the copy'n'paste stuff
		if(
			pcString->Compare( "v" ) == 0 ||
			pcString->Compare( "V" ) == 0 ||
			pcString->Compare( "c" ) == 0 ||
			pcString->Compare( "C" ) == 0 ||
			pcString->Compare( "x" ) == 0 ||
			pcString->Compare( "X" ) == 0
		)
		{
			TextView::KeyDown( pzString, pzRawString, nQualifiers );
		}
		return;
	}
}

